import 'dart:convert';
import 'dart:io';

import 'package:mockito/mockito.dart';
import 'package:payment_processors/credit_card.dart';
import 'package:payment_processors/culqi_request.dart';
import 'package:payment_processors/culqi_token_request.dart';
import 'package:payment_processors/culqi_token_response.dart';
import 'package:test/test.dart';
import 'package:http/http.dart' as http;

class MockHttpClient extends Mock implements http.Client {}

void main() {
  Map<String, String> headers;

  CulqiTokenRequest request;

  String jsonString;
  CreditCard card;
  //TODO: Make this work regardless of OS

  setUp(() async {
    card = CreditCard(
        cvv: 123, expMonth: 09, expYear: 2020, number: "4111111111111111");
    request = CulqiTokenRequest(card: card, email: "richard@piedpiper.com");
    File file = File('test\\fixtures\\token_generation_ok.json');
    jsonString = await file.readAsString();

    headers = {
      "Content-type": "application/json",
      "Authorization": "Bearer pk_test_5b455fdb2360787e"
    };
  });

  test('Checks a valid request', () async {
    final client = MockHttpClient();
    when(
      client.post(CulqiRequest.CULQI_TOKEN_URL,
          headers: headers, body: request.toJson()),
    ).thenAnswer((_) async => http.Response(jsonString, 200));

    expect(await CulqiRequest.getCulqiToken(client, request.toJson()),
        isA<CulqiTokenResponse>());
  });

  test('Checks an invalid request', () async {
    final client = MockHttpClient();
    when(
      client.post(CulqiRequest.CULQI_TOKEN_URL,
          headers: headers, body: request.toJson()),
    ).thenAnswer((_) async => http.Response("Not Found", 404));

    expect(
        CulqiRequest.getCulqiToken(client, request.toJson()), throwsException);
  });
}
